/*
blowfish.c:  C implementation of the Blowfish algorithm.

Copyright (C) 1997 by Paul Kocher and Bruce Schneier

Some modifications and extensions where made by Ralf David to provide
a function which en- and decrypt entire files and works under Palm OS
and Windows. I, Andreas Hofmeier, ported the code to Unix environment.

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   

*/



#ifndef __bfc_h__
#define __bfc_h__



#define E_NOT_SUPPORTED -1
#define E_NO_MEMORY -2
#define E_EOPEN -3
#define E_ECREATE -4
#define E_EABORTED -5
#define E_EREAD -6
#define E_SMALL_BUF -7
#define E_EWRITE -8

//#define BLOWFISHSLOW

#define _bfN   16



// Palm data types
typedef signed char		Int8;
typedef signed short		Int16;
typedef signed long		Int32;
typedef unsigned char	UInt8;
typedef unsigned short  UInt16;
typedef unsigned long   UInt32;
typedef char				Boolean;

#define true			 	1
#define false			 	0


#define MemHandle 			   void*
#define MemHandleNew(size)    malloc(size)
#define MemHandleLock(memH)   memH
#define MemHandleUnlock(memH) ;
#define MemHandleFree(memH)   free(memH)


#define chrSpace	              32
#define errNone		            0
#define appError	          0x8000
#define appStatCancel       0x8000
#define BFMAXKEYBYTES           56     /* 448 bits */


// Read an Int32 from Motorola-order
#define rm32(mp) (((UInt32)(((UInt8*)(mp))[0]) << 24 ) | \
   	            ((UInt32)(((UInt8*)(mp))[1]) << 16 ) | \
		            ((UInt32)(((UInt8*)(mp))[2]) <<  8 ) | \
                  ((UInt32)(((UInt8*)(mp))[3]) 	   )   )

// Read an Int16 from Motorola-order
#define rm16(mp) (((UInt16)(((UInt8*)(mp))[0]) << 8 ) | \
		  ((UInt16)(((UInt8*)(mp))[1]) 	    )	  )

// Read an Int32 from Intel-order
#define ri32(ip) (((UInt32)(((UInt8*)(ip))[3]) << 24 ) | \
					   ((UInt32)(((UInt8*)(ip))[2]) << 16 ) | \
					   ((UInt32)(((UInt8*)(ip))[1]) <<  8 ) | \
					   ((UInt32)(((UInt8*)(ip))[0]) 	  	  )	)

// Read an Int16 from Intel-order
#define ri16(ip) (((UInt16)(((UInt8*)(ip))[1]) << 8 ) | \
					   ((UInt16)(((UInt8*)(ip))[0]) 	    )	  )


// Store an Int32 in Motorola-order
#define sm32(mp,x) {  ((UInt8*)(mp))[3] = (UInt8) (x)     ; \
   				   	 ((UInt8*)(mp))[2] = (UInt8)((x)>> 8); \
	   		   		 ((UInt8*)(mp))[1] = (UInt8)((x)>>16); \
		      			 ((UInt8*)(mp))[0] = (UInt8)((x)>>24); }

// Store an Int32 in Motorola-order
#define sm16(mp,x) { ((UInt8*)(mp))[1] = (UInt8) (x)     ; \
					      ((UInt8*)(mp))[0] = (UInt8)((x)>> 8); }

// Store an Int32 in Intel-order
#define si32(ip,x) { ((UInt8*)(ip))[0] = (UInt8) (x)     ; \
					      ((UInt8*)(ip))[1] = (UInt8)((x)>> 8); \
					      ((UInt8*)(ip))[2] = (UInt8)((x)>>16); \
					      ((UInt8*)(ip))[3] = (UInt8)((x)>>24); }

// Store an Int32 in Intel-order
#define si16(ip,x) { ((UInt8*)(ip))[0] = (x)    ; \
					      ((UInt8*)(ip))[1] = (x)>> 8; }



#define swap16(n) (((((UInt16) n) << 8) & 0xFF00) | 	  \
				    	 ((((UInt16) n) >> 8) & 0x00FF)	  	  )

#define swap32(n) (((((UInt32) n) << 24) & 0xFF000000) | \
						 ((((UInt32) n) <<  8) & 0x00FF0000) | \
						 ((((UInt32) n) >>  8) & 0x0000FF00) | \
						 ((((UInt32) n) >> 24) & 0x000000FF)   )



	void  *BlowfishInitKey(void* keyP, UInt8 keySize);
        void   Blowfish_Init(void *ctxP, unsigned char *key, int keyLen);
	void   BlowfishDeleteKey(void *ctx);
	void   Blowfish_Encrypt(void *ctx, UInt32 *xlP, UInt32 *xrP);
	void   Blowfish_Decrypt(void *ctx, UInt32 *xlP, UInt32 *xrP);
	UInt32 BlowfishEncryptBuffer(void *ctx, void *bufferP, UInt32 size);
	UInt32 BlowfishDecryptBuffer(void *ctx, void *bufferP, UInt32 size);
	UInt32 BlowfishCBCDecryptBuffer(void *ctx, void *bufferP, UInt32 size, UInt32 *leftCVp, UInt32 *rightCVp);
	UInt32 BlowfishCBCEncryptBuffer(void *ctx, void *bufferP, UInt32 size, UInt32 *leftCVp, UInt32 *rightCVp);
	UInt16 BlowfishSelftest(void);
	UInt16 BlowfishCheckKey(char *keyP);
   UInt32 Random32(void);


     typedef struct
{  UInt32     P[16 + 2];
       UInt32     S[4][256];
       UInt32    crc;
}     BLOWFISH_CTX;


#endif // __bfc_h__

