#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bfc.h"



phelp() {
  printf("\n\n.bfc (used by UniCMD palm OS file mgr) en- and decryption utility.\n\n");
  printf("Usage: bfcoder [-e] [-d] [-s] [-overwrite] [-p <password>] <infile> [outfile]\n");
  printf("          -e: encrypt infile\n");
  printf("          -d: decrypt infile\n");
  printf("          -s: conduct selftest (of the encryption code).\n");
  printf("          -p: specify the password. (You will be ask to enter it if this option\n"
	 "              is missing.)\n");
  //  printf("          -overwrite: overwrite the infile with * (0x2a) after ancryption.\n");
  printf("          You can use \"-\" for stdin/stdout instead of infile/outfile.\n\n");
  printf("Examples: bfcoder foo.bfc | less  - Decode file foo.bfc and show the cleartext.\n" \
	 "                                    No data is written to the harddrive.\n");
  printf("          bfcoder foo.bfc         - Decrypt file foo.bfc to foo.\n");
  printf("          bfcoder foo             - Encrypt file foo to foo.bfc.\n");
	 



}


char blacky_ver_info[] = "bfcoder - bfc en- and decryption utility v0.01";


main(int argc, char **argv) {
  BLOWFISH_CTX ctx;

  int force_encode = 0;
  int force_decode = 0;
  int do_selftest = 0;
  int force_overwrite = 0;// not implemented
  char *in = NULL;
  char *out = NULL;
  char *password = NULL;
  int quiet = 0;

  int i;

  for (i = 1; i < argc; i++) {
    if (strcasecmp(argv[i], "-d") == 0) {
      force_decode = 1;
    } else if (strcasecmp(argv[i], "-e") == 0) {
      force_encode = 1;
    } else if (strcasecmp(argv[i], "-s") == 0) {
      do_selftest = 1;
    } else if (strncasecmp(argv[i], "-p", 2) == 0) {
      if (argv[i][2] != 0) {
	password = argv[i] + 2;
      } else {
	i++;
	if (i >= argc) {
	  fprintf(stderr, "Parameter Error: Password is missing after -p.\n");
	  phelp();
	  exit(1);
	}
	password = argv[i];
      }
    } else if (strcasecmp(argv[i], "-overwrite") == 0) {
      force_overwrite = 1;
    } else if (in == NULL) {
      in = argv[i];
    } else if (out == NULL) {
      out = argv[i];
    } else {
      fprintf(stderr, "Unknown option: %s\n", argv[i]);
      phelp();
      exit(1);
    }
  }

  if ((out != NULL) && (strcmp(out, "-") == 0)) {
    quiet = 1;
  }

  if (do_selftest) {
    if (!quiet)
      printf("Selftest: %s\n", (BlowfishSelftest() == errNone) ? "succeed." : "failed.");
  }

  // check given input file
  if (in == NULL) {
    fprintf(stderr, "Parameter Error: no input file specified.\n");
    phelp();
    exit(1);
  }
  {
    FILE *f;

    f = fopen(in, "r");
    if (f == NULL) {
      fprintf(stderr, "Cannot open input file %s.\nExitting...\n", in);
      exit(1);
    }
    fclose(f);
  }


  // build output filename
  if (out == NULL) {
    #define BUFSIZE 1024
    static char outname[BUFSIZE];

    int len = strlen(in);

    if (strcmp(in, "-") == 0) {
      fprintf(stderr, "Please specify an output file.\n");
      phelp(); 
      exit(1);
    }

    if (strcasecmp(in + len - 4, ".bfc") == 0) {
      if (force_encode) {
	fprintf(stderr, "WARNING: Do you really want to encrypt an already encrypted file?\nIf so, please rename the input file and remove .bfc.\nExitting...\n");
	exit(1);
      }
      force_decode = 1;
      strcpy(outname, in);
      outname[len - 4] = 0;
      out = outname;
    } else if (force_decode) {
      strcpy(outname, in);
      strcat(outname, ".out");
      out = outname;
    } else {
      force_encode = 1;
      strcpy(outname, in);
      strcat(outname, ".bfc");
      out = outname;
    }
  }

  // check if output file exists already
  {
    FILE *f;

    f = fopen(out, "r");
    if (f != NULL) {
      fprintf(stderr, "Output file %s already exist.\nExitting...\n", out);
      fclose(f);
      exit(1);
    }
  }

  if (strcmp(out, "-") == 0) {
    if (strcasecmp(in + strlen(in) - 4, ".bfc") == 0) {
      force_decode = 1;
    }
  }

  if (((force_encode == 1) && (force_decode == 1)) ||
      ((force_encode == 0) && (force_decode == 0))) {
    fprintf(stderr, "What do you want? Encrypt or decrypt? Please give me a hint.\n");
    phelp();
    exit(1);
  }


  // enter password
  if (password == NULL) {
    #define maxlen 512
    static char p0[maxlen];
    char p1[maxlen];
    system("stty -echo");
    fprintf(stderr, "Please enter the password (it is not shown): ");
    fgets(p0, maxlen -1, stdin);
    fprintf(stderr, "\n");
    fprintf(stderr, "Please enter the password again (confirmation): ");
    fgets(p1, maxlen -1, stdin);
    fprintf(stderr, "\n");
    system("stty echo");

    if (strcmp(p0, p1) != 0) {
      fprintf(stderr, "Error: Passwords do not match.\nExitting...\n");
      exit(1);
    }

    password = p0;
  }

  if (BlowfishCheckKey(password) != 0) {
    fprintf(stderr, "Warning: Invalid password lenght\n");
    //    exit(1);
  }

  if (!quiet) {
    if (force_decode) {
      printf("decrypt:\n");
    }
    if (force_encode) {
      printf("encrypt:\n");
    }
    printf("  IN:  %s\n", in);
    printf("  OUT: %s\n", out);
  }

  Blowfish_Init(&ctx, password, strlen(password));
  FileCopy(force_decode, &ctx, in, out);

  if (!quiet)
    printf("Warning: This program do not do any integrity checking of the data!\n");
}
